//
// Misc routines used in testing graphics functionality
//

#include "stdafx.h"

#include <DDraw.h>
#include <ISTBTDDExt.h>
#include "d3dtypes.h"					// Include for RGBA_MAKE macro

#include "tulsa sample app.h"
#include "mainfrm.h"
#include "CardDlg.h"
#include "config.h"
#include "InitCardDlg.h"
#include "FrameListDlg.h"
#include "dibapi.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define BAR_COLOR(y, cb, cr)	(((cb) << 24) | ((y) << 16) | ((cr) << 8) | ((y) << 0))
#define YUV_SAMPLE(y1, y0, cb, cr)    (((cb) << 16) | ((y1) << 24) | ((cr) << 0) | ((y0) << 8))
static DWORD YUVBarColors[] = {
	BAR_COLOR(180, 128, 128),
	BAR_COLOR(162, 44, 142),
	BAR_COLOR(131, 156, 44),
	BAR_COLOR(112, 72, 58),
	BAR_COLOR(84, 184, 198),
	BAR_COLOR(65, 100, 212),
	BAR_COLOR(35, 212, 114),
	BAR_COLOR(16, 128, 128)
};

	// copy bitmap to specified surface - assumes 32-bit color surface
void CCardDlg::CopyBitmap(DDSURFACEDESC* pSurface, int BitmapId, DWORD AlphaStart, ALPHA_EFFECT AlphaEffect, BOOL bUpdateAlphaChannel) 
{
	int x, y;
	double AlphaRamp, AlphaDelta;
	DWORD AlphaToUse;
	CBitmap Bmp;
	DWORD Stride = pSurface->lPitch;
	DWORD Height = pSurface->dwHeight;
	DWORD Width = pSurface->dwWidth;
	DWORD BytesPerPixel = pSurface->ddpfPixelFormat.dwRGBBitCount/8;	// only 16 & 32 supported
	LPVOID lpSurface = pSurface->lpSurface;

	if (Bmp.LoadBitmap(BitmapId) != 0) {
		BITMAP bm;
					// Get bitmap dimensions
	    GetObject(Bmp, sizeof(bm), (LPSTR)&bm);
		Width = min((int)Width, bm.bmWidth);
		Width *= 4;
		Height = min((int)Height, bm.bmHeight);
	
		LPBITMAPINFO pDIBHdr;
		BYTE *pDIBBits; 
		
		HDIB hDib = DibFromBitmap((HBITMAP)Bmp, BI_RGB, 32, NULL);
		if (hDib != NULL) {
		    pDIBHdr = (LPBITMAPINFO)GlobalLock(hDib);
			pDIBBits = (BYTE*)FindDIBBits((char*)pDIBHdr);

			AlphaRamp = 0;

			for (y = 0; y < (int)Height; y++) {
				DWORD SrcRowOffset = (Height - y - 1)*bm.bmWidth*4;
				DWORD DestRowOffset = y*Stride;

				if (bUpdateAlphaChannel) {
					switch (AlphaEffect) {
					case ALPHA_EFFECT_LTOR:
						AlphaDelta = ((double)255 - (double)AlphaStart)/(double)bm.bmWidth;
						AlphaRamp = 0;
						break;
					case ALPHA_EFFECT_RTOL:
						AlphaDelta = -1*((double)AlphaStart - (double)0)/(double)bm.bmWidth;
						AlphaRamp = AlphaStart;
						break;
					case ALPHA_EFFECT_TTOB:
					case ALPHA_EFFECT_BTOT:
						//not supported yet...
					default:
					case ALPHA_EFFECT_NONE:
						AlphaDelta = 0.0;
						break;
					}
				}

				for (x = 0; x < (int)Width; x += 4) {
					DWORD PixColor = *(DWORD*)((DWORD)pDIBBits + SrcRowOffset + x);

					if (bUpdateAlphaChannel) {
						if (PixColor == 0xff) {
							AlphaToUse = (DWORD)AlphaRamp;
						} else {
							AlphaToUse = 0xff;
						}
						AlphaRamp += AlphaDelta;

						*(DWORD*)((DWORD)lpSurface + DestRowOffset + x) = PixColor | (AlphaToUse << 24);
					} else {
						DWORD CurPixel = *(DWORD*)((DWORD)lpSurface + DestRowOffset + x);
						*(DWORD*)((DWORD)lpSurface + DestRowOffset + x) = PixColor | (CurPixel & 0xff000000);

					}
				}
			}
		    GlobalUnlock(hDib);
	        GlobalFree(hDib);
		}
		Bmp.DeleteObject();
	}
}


	// copy bitmap to specified surface - assumes 32-bit color surface
void CCardDlg::CopyBitmapFromFile(DDSURFACEDESC* pSurface, CGraphicsContent *pGraphicsContent, BOOL bUpdateAlphaChannel) 
{
	int x, y, xs;
	double AlphaRamp, AlphaDelta;
	DWORD AlphaToUse;
	CBitmap Bmp;
	DWORD Stride = pSurface->lPitch;
	DWORD Height = pSurface->dwHeight;
	DWORD Width = pSurface->dwWidth;
	DWORD BytesPerPixel = pSurface->ddpfPixelFormat.dwRGBBitCount/8;	// only 16 & 32 supported
	LPVOID lpSurface = pSurface->lpSurface;

	HDIB hDib;
	BITMAPINFOHEADER bi;
	if ((hDib = OpenDIB(pGraphicsContent->m_Filename, &bi)) != NULL) {
		Width = min((int)Width, bi.biWidth);
		Width *= 4;
		Height = min((int)Height, bi.biHeight);
	
		LPBITMAPINFO pDIBHdr;
		BYTE *pDIBBits; 

		DWORD BytesPerPixel = bi.biBitCount/8;
		DWORD BytesPerRow = BytesPerPixel*bi.biWidth;
											// DWORD align row
		if (BytesPerRow % 4) {
			BytesPerRow += 4 - (BytesPerRow % 4);
		}

	    pDIBHdr = (LPBITMAPINFO)GlobalLock(hDib);
		pDIBBits = (BYTE*)FindDIBBits((char*)pDIBHdr);
	
		DWORD AlphaMask;
		if (pGraphicsContent->m_AlphaSource == ALPHA_SRC_GENERATE) {
			AlphaRamp = pGraphicsContent->m_StartingAlpha;
			AlphaMask = 0xff000000;
		} else {
			AlphaRamp = 0;
			AlphaMask = 0x00000000;
		}	

		for (y = 0; y < (int)Height; y++) {
			DWORD SrcRowOffset = (Height - y - 1)*BytesPerRow;
			DWORD DestRowOffset = y*Stride;

			if (bUpdateAlphaChannel && 
				pGraphicsContent->m_AlphaSource == ALPHA_SRC_GENERATE) {

				switch (pGraphicsContent->m_AlphaEffect) {
				case ALPHA_EFFECT_LTOR:
					AlphaDelta = ((double)255 - (double)pGraphicsContent->m_StartingAlpha)/(double)bi.biWidth;
					AlphaRamp = 0;
					break;
				case ALPHA_EFFECT_RTOL:
					AlphaDelta = -1*((double)pGraphicsContent->m_StartingAlpha - (double)0)/(double)bi.biWidth;
					AlphaRamp = pGraphicsContent->m_StartingAlpha;
					break;
				case ALPHA_EFFECT_TTOB:
				case ALPHA_EFFECT_BTOT:
					//not supported yet...
				default:
				case ALPHA_EFFECT_NONE:
					AlphaDelta = 0.0;
					AlphaRamp = pGraphicsContent->m_StartingAlpha;
					break;
				}
			}

			for (x = 0, xs = 0; x < (int)Width; x += 4, xs += BytesPerPixel) {
				DWORD PixColor;
				if (BytesPerPixel == 4) {
					PixColor = *(DWORD*)((DWORD)pDIBBits + SrcRowOffset + xs);
				} else if (BytesPerPixel == 3) {
					PixColor = ((DWORD)*(pDIBBits + SrcRowOffset + xs)) |
						(((DWORD)*(pDIBBits + SrcRowOffset + xs + 1)) << 8) |
						(((DWORD)*(pDIBBits + SrcRowOffset + xs + 2)) << 16);
				} else {				// Not supported
					PixColor = 0;		
				}

				PixColor &= ~AlphaMask;

				if (bUpdateAlphaChannel) {
					AlphaToUse = (DWORD)AlphaRamp;
					AlphaRamp += AlphaDelta;
														// Temporary - assume magenta is a color key 
					if (PixColor == RGB(255, 0, 255)) {
						AlphaToUse = 0xff;	
					}
					*(DWORD*)((DWORD)lpSurface + DestRowOffset + x) = PixColor | (AlphaToUse << 24);
				} else {
					DWORD CurPixel = *(DWORD*)((DWORD)lpSurface + DestRowOffset + x);
					*(DWORD*)((DWORD)lpSurface + DestRowOffset + x) = PixColor | (CurPixel & 0xff000000);
				}
			}
		}
		GlobalUnlock(hDib);
	    GlobalFree(hDib);
	}
}



#define NUM_COLORS 9

DWORD Colors[NUM_COLORS] = {
	0x00000000, 
	0x000000ff,
	0x0000ff00,
	0x00ff0000,
	0xff000000,
	0x0000ffff,
	0x00ffff00,
	0x00ff00ff,
	0x00ffffff
};

void CCardDlg::DrawTestPattern(DDSURFACEDESC* pSurface, COLORREF color)
{
	DWORD Stride = pSurface->lPitch;
	DWORD Height = pSurface->dwHeight;
	DWORD Width = pSurface->dwWidth;
	DWORD BytesPerPixel = pSurface->ddpfPixelFormat.dwRGBBitCount/8;	// only 16 & 32 supported
	LPVOID lpSurface = pSurface->lpSurface;
	DWORD Offset;
	DWORD FillValue;

	if (BytesPerPixel == 2) {
		FillValue = ((GetRValue(color) & 0xf8) << 8) |
			((GetGValue(color) & 0xfc) << 3) |
			((GetBValue(color) & 0xf8) >> 3);
	} else {
		FillValue = color;
	}

	FillValue = 0;
	int ColorIndex = 0;
	BYTE TempColor;

	for (int line = 0; line < (int)Height; line++) {

		TempColor = 0;

		if ((line % 40) == 0) {
			FillValue = Colors[ColorIndex++];
			if (ColorIndex > NUM_COLORS) {
				ColorIndex = 0;
			}
		}
		Offset = line*Stride;
		for (int x = 0; x < (int)Width*(int)BytesPerPixel; x += (int)BytesPerPixel) {
			if (BytesPerPixel == 2) {
				*(WORD*)((DWORD)lpSurface + Offset + x) = (WORD)FillValue;
			} else {
//				*(DWORD*)((DWORD)lpSurface + Offset + x) = FillValue;
				*(DWORD*)((DWORD)lpSurface + Offset + x) = (TempColor++) << 16;

			}
		}
	}
}



void CCardDlg::FillSurface(LPDIRECTDRAWSURFACE lpDDSurface, DWORD color)
{
	DDSURFACEDESC SurfaceDesc;
	if (lpDDSurface->Lock(NULL, &SurfaceDesc, 0, NULL) != DD_OK) {
		return;
	}

	DWORD Stride = SurfaceDesc.lPitch;
	DWORD Height = SurfaceDesc.dwHeight;
	DWORD Width = SurfaceDesc.dwWidth;
	DWORD BytesPerPixel = SurfaceDesc.ddpfPixelFormat.dwRGBBitCount/8;	// only 16 & 32 supported
	LPVOID lpSurface = SurfaceDesc.lpSurface;
	DWORD Offset;
	DWORD FillValue;

	if (BytesPerPixel == 2) {
		FillValue = ((GetRValue(color) & 0xf8) << 8) |
			((GetGValue(color) & 0xfc) << 3) |
			((GetBValue(color) & 0xf8) >> 3);
	} else {
		FillValue = color;
	}

	for (int line = 0; line < (int)Height; line++) {
		Offset = line*Stride;
		for (int x = 0; x < (int)(Width*BytesPerPixel); x += BytesPerPixel) {
			if (BytesPerPixel == 2) {
				*(WORD*)((DWORD)lpSurface + Offset + x) = (WORD)FillValue;
			} else {
				*(DWORD*)((DWORD)lpSurface + Offset + x) = FillValue;
			}
		}
	}

	lpDDSurface->Unlock(NULL);

}


void CCardDlg::DrawYUVColorBars(LPDIRECTDRAWSURFACE lpDDSurface)
{
	DDSURFACEDESC SurfaceDesc;
	if (lpDDSurface->Lock(NULL, &SurfaceDesc, 0, NULL) != DD_OK) {
		return;
	}

	DWORD Stride = SurfaceDesc.lPitch;
	DWORD Height = 504; 
	DWORD Width = SurfaceDesc.dwWidth;
	DWORD BytesPerPixel = SurfaceDesc.ddpfPixelFormat.dwRGBBitCount/8;	// only 16 & 32 supported
	LPVOID lpSurface = SurfaceDesc.lpSurface;
	DWORD Offset;
	DWORD FillValue;
	int ColorBarWidth = Width*2/8;

	for (int line = 0; line < (int)Height; line++) {
		Offset = line*Stride;

		int z = 0;
		for (int x = 0; x < (int)Width*2; x += 4, z++) {
			FillValue = YUVBarColors[x/ColorBarWidth];
			if (z > line) {
				*(DWORD*)((DWORD)lpSurface + Offset + x) = (((0x80) << 24) | ((0) << 16) | ((0x80) << 8) | ((0xff) << 0));
			} else {
				*(DWORD*)((DWORD)lpSurface + Offset + x) = (((0x80) << 24) | ((0) << 16) | ((0x80) << 8) | ((0) << 0));
			}
		}
	}
	lpDDSurface->Unlock(NULL);

}


void CCardDlg::ShowTitleOverlay(void)
{
	HRESULT ddrval;
	DDSURFACEDESC   ddsd;
	DDCOLORKEY DDColorKey;

	if (lpDDSTitle) {
		return;
	}

	LPTULSASURFACEEXT lpTulsaSurfaceExt;

	HRESULT result = CoCreateInstance(CLSID_STBTDDExtension, NULL, 
		CLSCTX_INPROC, IID_ISTBTDDSurfaceExt, (LPVOID *)&lpTulsaSurfaceExt);
	if (result != DD_OK) {
		return;
	}

    INIT_DIRECTDRAW_STRUCT(ddsd);
	ddsd.dwHeight = pTitleOvly->m_Size.cy;
	ddsd.dwWidth = pTitleOvly->m_Size.cx;
	ddsd.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	ddsd.ddpfPixelFormat.dwFlags = DDPF_RGB;
	ddsd.ddpfPixelFormat.dwRGBBitCount = 32;
    ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OVERLAY;

										// Create an overlay surface for graphics
	ddrval = lpDD2RGB->CreateSurface(&ddsd, &lpDDSTitle, NULL);
	if (ddrval == DD_OK) {
										// Assign surface to extend
		lpTulsaSurfaceExt->AssignSurface(lpDDSTitle);
										// Specify the handling for the alpha channel
		SURFACEDESCEXT SurfaceDescExt;
		INIT_DIRECTDRAW_STRUCT(SurfaceDescExt);
		SurfaceDescExt.dwFlags = 0;
		SurfaceDescExt.bPreserveDestAlpha = !pTitleOvly->m_bWriteAlpha;
		lpTulsaSurfaceExt->SetSurfaceDescExt(&SurfaceDescExt);

										// Clear the surface
		FillSurface(lpDDSTitle, RGBA_MAKE(0x80, 0x0, 0x0, 0x80));

										// Copy bitmap onto surface and display
										//  with no blending or colorkeying
		DDSURFACEDESC SurfaceDesc;
		if (lpDDSTitle->Lock(NULL, &SurfaceDesc, 0, NULL) == DD_OK) {
										// Copy bitmap to surface
			if (pTitleOvly->m_Content.m_Source == SRC_BMP_FILE) {
				CopyBitmapFromFile(&SurfaceDesc, &pTitleOvly->m_Content, TRUE);
			}
			lpDDSTitle->Unlock(NULL);
		}

		RECT SrcRect, DestRect;
		SrcRect.left = 0;
		SrcRect.top = 0;
		SrcRect.right = SrcRect.left + pTitleOvly->m_Size.cx - 1;
		SrcRect.bottom = SrcRect.top + pTitleOvly->m_Size.cy - 1;
		DestRect.left = pTitleOvly->m_Position.x;
		DestRect.top = pTitleOvly->m_Position.y;
		DestRect.right = DestRect.left + pTitleOvly->m_Size.cx - 1;
		DestRect.bottom = DestRect.top + pTitleOvly->m_Size.cy - 1;

		TitleOverlaySize = pTitleOvly->m_Size;

		if (pTitleOvly->m_bCkEnable) {
										// Show bitmap with color keying enabled
			DDColorKey.dwColorSpaceHighValue = DDColorKey.dwColorSpaceLowValue = pTitleOvly->m_CkColor;
			lpDDSTitle->SetColorKey(DDCKEY_SRCOVERLAY, &DDColorKey);

			lpDDSTitle->UpdateOverlay(&SrcRect, lpDDSPrimaryRGB, &DestRect, 
				DDOVER_SHOW | DDOVER_KEYSRC, NULL);
		} else {
			lpDDSTitle->UpdateOverlay(&SrcRect, lpDDSPrimaryRGB, &DestRect, 
				DDOVER_SHOW | (pTitleOvly->m_bBlend? DDOVER_ALPHASRC: 0) | 
				(pTitleOvly->m_bInvBlend? DDOVER_ALPHASRCNEG: 0), NULL);
		}

		if (TitleOvlyZOrderFlags) {
			lpDDSTitle->UpdateOverlayZOrder(TitleOvlyZOrderFlags & 0xfffffff, NULL);
			TitleOvlyZOrderFlags = 0;
		}
	} else {
		AfxMessageBox("Could not create test overlay surface.");
	}
	lpTulsaSurfaceExt->Release();

}

void CCardDlg::DeleteTitleOverlay(void)
{
	if (lpDDSTitle) {
		lpDDSTitle->Release();
		lpDDSTitle = NULL;
	}
}

void CCardDlg::UpdateTitleOverlay(void)
{
	DDCOLORKEY DDColorKey;

	if (lpDDSTitle) {

		LPTULSASURFACEEXT lpTulsaSurfaceExt;

		HRESULT result = CoCreateInstance(CLSID_STBTDDExtension, NULL, 
			CLSCTX_INPROC, IID_ISTBTDDSurfaceExt, (LPVOID *)&lpTulsaSurfaceExt);
		if (result != DD_OK) {
			return;
		}
										// Assign surface to extend
		lpTulsaSurfaceExt->AssignSurface(lpDDSTitle);
										// Specify the handling for the alpha channel
		SURFACEDESCEXT SurfaceDescExt;
		INIT_DIRECTDRAW_STRUCT(SurfaceDescExt);
		SurfaceDescExt.dwFlags = 0;
		SurfaceDescExt.bPreserveDestAlpha = !pTitleOvly->m_bWriteAlpha;
		lpTulsaSurfaceExt->SetSurfaceDescExt(&SurfaceDescExt);

		lpTulsaSurfaceExt->Release();


		RECT SrcRect, DestRect;
		SrcRect.left = 0;
		SrcRect.top = 0;
		SrcRect.right = SrcRect.left + TitleOverlaySize.cx - 1;
		SrcRect.bottom = SrcRect.top + TitleOverlaySize.cy - 1;
		DestRect.left = pTitleOvly->m_Position.x;
		DestRect.top = pTitleOvly->m_Position.y;
		DestRect.right = DestRect.left + pTitleOvly->m_Size.cx - 1;
		DestRect.bottom = DestRect.top + pTitleOvly->m_Size.cy - 1;

		if (pTitleOvly->m_bCkEnable) {
										// Show bitmap with color keying enabled
			DDColorKey.dwColorSpaceHighValue = DDColorKey.dwColorSpaceLowValue = pTitleOvly->m_CkColor;
			lpDDSTitle->SetColorKey(DDCKEY_SRCOVERLAY, &DDColorKey);

			lpDDSTitle->UpdateOverlay(&SrcRect, lpDDSPrimaryRGB, &DestRect, 
				DDOVER_SHOW | DDOVER_KEYSRC, NULL);
		} else {
			lpDDSTitle->UpdateOverlay(&SrcRect, lpDDSPrimaryRGB, &DestRect, 
				DDOVER_SHOW | (pTitleOvly->m_bBlend? DDOVER_ALPHASRC: 0) | 
				(pTitleOvly->m_bInvBlend? DDOVER_ALPHASRCNEG: 0), NULL);
		}
		if (TitleOvlyZOrderFlags) {
			lpDDSTitle->UpdateOverlayZOrder(TitleOvlyZOrderFlags & 0xfffffff, NULL);
			TitleOvlyZOrderFlags = 0;
		}
	}
}


#define TEST_HEIGHT 480
#define TEST_WIDTH 720
#define MAX_OVERLAYS	10

void CCardDlg::TestSurfaceAlloc(void)
{
	HRESULT ddrval;
	LPDIRECTDRAWSURFACE lpDDS[MAX_OVERLAYS];
	DDSURFACEDESC   ddsd;
	char msg[100];
    INIT_DIRECTDRAW_STRUCT(ddsd);

	return;

	ddsd.dwHeight = TEST_HEIGHT;
	ddsd.dwWidth = TEST_WIDTH;
	ddsd.ddpfPixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	ddsd.ddpfPixelFormat.dwFlags = DDPF_RGB;
	ddsd.ddpfPixelFormat.dwRGBBitCount = 32;
    ddsd.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT | DDSD_PIXELFORMAT;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OVERLAY;
										// Create an overlay surface for graphics
	for (int i = 0; i < MAX_OVERLAYS; i++) {
		ddrval = lpDD2RGB->CreateSurface(&ddsd, &lpDDS[i], NULL);
		if (ddrval != DD_OK) {
			sprintf(msg, "Failed to create surface #d.", i);
			AfxMessageBox(msg);
			lpDDS[i] = NULL;
		}
	}
	
	for (i = 0; i < MAX_OVERLAYS && lpDDS[i] != NULL; i++) {
										// Copy bitmap onto surface and display
										//  with no blending or colorkeying
		DDSURFACEDESC SurfaceDesc;
		if (lpDDS[i]->Lock(NULL, &SurfaceDesc, 0, NULL) == DD_OK) {
										// Copy bitmap to surface
			CopyBitmapFromFile(&SurfaceDesc, &PriSurfaceContents, TRUE);
		}

		RECT SrcRect, DestRect;
		SrcRect.left = 0;
		SrcRect.top = 0;
		SrcRect.right = SrcRect.left + TEST_WIDTH - 1;
		SrcRect.bottom = SrcRect.top + TEST_HEIGHT - 1;
		DestRect.left = 0;
		DestRect.top = 0;//150;//25;
		DestRect.right = DestRect.left + TEST_WIDTH - 1;
		DestRect.bottom = DestRect.top + TEST_HEIGHT - 1;

		lpDDS[i]->UpdateOverlay(&SrcRect, lpDDSPrimaryRGB, &DestRect, 
			DDOVER_SHOW | DDOVER_ALPHASRC/* | DDOVER_ALPHASRCNEG*/, NULL);

		lpDDS[i]->UpdateOverlayZOrder(DDOVERZ_SENDTOFRONT, NULL);
	}
}

void CCardDlg::GetAlphaSettings(ALPHA_OPTIONS* pAlphaOptions)
{
	*pAlphaOptions = AlphaOptions;
}

void CCardDlg::ApplyAlphaSettings(BOOL bUseBackbuffer)
{
	LPDIRECTDRAWSURFACE lpDDS;
	DDSURFACEDESC SurfaceDesc;

	if (lpDDSPrimaryRGB == NULL) {
		return;
	}
	
	if (bUseBackbuffer) {
		DDSCAPS ddscaps;
		ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
		if (lpDDSPrimaryRGB->GetAttachedSurface(&ddscaps, &lpDDS) != DD_OK) {
			return;
		}
	} else {
		lpDDS = lpDDSPrimaryRGB;
	}

	LPTULSASURFACEEXT lpTulsaSurfaceExt;

	HRESULT result = CoCreateInstance(CLSID_STBTDDExtension, NULL, 
		CLSCTX_INPROC, IID_ISTBTDDSurfaceExt, (LPVOID *)&lpTulsaSurfaceExt);
	if (result != DD_OK) {
		return;
	}
										// Assign surface to extend
	lpTulsaSurfaceExt->AssignSurface(lpDDSPrimaryRGB);
										// Specify the handling for the alpha channel
	SURFACEDESCEXT SurfaceDescExt;
	INIT_DIRECTDRAW_STRUCT(SurfaceDescExt);
	SurfaceDescExt.dwFlags = 0;
	SurfaceDescExt.bPreserveDestAlpha = FALSE;	//N/A for primary surface
	SurfaceDescExt.bInvertAlpha = AlphaOptions.bInvertAlpha;
	lpTulsaSurfaceExt->SetSurfaceDescExt(&SurfaceDescExt);

	lpDDSPrimaryRGB->GetSurfaceDesc(&SurfaceDesc);

	FILLSURFACECMD FillCmd;
	FillCmd.dwFlags = FSC_ENABLE;
	FillCmd.dwExecCount = 0x00000002;
	FillCmd.dwMask = 0xff000000;
	FillCmd.dwRed = 0;		
	FillCmd.dwAlpha = AlphaOptions.AlphaValue << 16;			
	FillCmd.dwGreen = 0;			
	FillCmd.dwBlue = 0;			
	FillCmd.dwRedDx = 0;			
	FillCmd.dwAlphaDx = 0;		
	FillCmd.dwGreenDx = 0;		
	FillCmd.dwBlueDx = 0;		
	FillCmd.dwRedDy = 0;			
	FillCmd.dwAlphaDy = 0;		
	FillCmd.dwGreenDy = 0;		
	FillCmd.dwBlueDy = 0;		
	FillCmd.dwRedDvert = 0;		
	FillCmd.dwAlphaDvert = 0;	
	FillCmd.dwGreenDvert = 0;	
	FillCmd.dwBlueDvert = 0;		

	if (AlphaOptions.bVertGradient) {
		FillCmd.dwAlphaDy = 0x10000*AlphaOptions.AlphaValue/SurfaceDesc.dwHeight*2;
		AlphaOptions.AlphaValue = 0;
	}

	if (AlphaOptions.bHorzGradient) {
		FillCmd.dwAlphaDx = 0x10000*AlphaOptions.AlphaValue/SurfaceDesc.dwWidth;
		AlphaOptions.AlphaValue = 0;
	}

	if (AlphaOptions.bTransitionAlpha) {
		if (AlphaOptions.TransitionVerticals == 0) {
			AlphaOptions.TransitionVerticals = 1;
		}
		FillCmd.dwAlphaDvert = (0x10000*(AlphaOptions.AlphaValue - m_bLastAlpha) + 0x8000)/AlphaOptions.TransitionVerticals;	
		FillCmd.dwExecCount = AlphaOptions.TransitionVerticals;
		FillCmd.dwAlpha = m_bLastAlpha << 16;			
	} else {
		FillCmd.dwAlpha = AlphaOptions.AlphaValue << 16;
	}

	m_bLastAlpha = AlphaOptions.AlphaValue;

	lpTulsaSurfaceExt->FillSurface(&FillCmd);

	lpTulsaSurfaceExt->Release();
}



